/* ========================================================================= *
 * analyzer.c -- analysis routines for EZ-pipe.  
 *
 * Copyright (C) 2011 Dwight Barkley                 
 * ========================================================================= */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "pipe.h"
#include "analyzer.h"

#define HISTORY 0		/* if 1 write history slices (can be large) */

static FILE *energy_file;
static FILE *history_file;

/* ========================================================================= */

int Analyze (int istep)
{
  /* integrated energy */
  double  q_tot = 0.;
  for (int i = 1; i <= nx; i++) {
    q_tot += q[i];
  }
  fprintf (energy_file, "%d %.5f\n", istep, q_tot);

  /* history slices */
  if (HISTORY) {
    for (int i = 1; i <= nx; i++) {
      fprintf (history_file, "%d %.5f %.5f\n", istep, u[i], q[i]);
    }
  }

  return 0;
}

/* ========================================================================= */

void Analyze_ini (void)
{
  energy_file = fopen ("energy.dat", "w");
  history_file = fopen ("history.dat", "w");

  fprintf (energy_file, "# Parameters: R, beta, gamma, epsilon1, epsilon2: ");
  fprintf (energy_file, "%g, %g, %g, %g, %g\n",
	   R_param, beta, gamma, epsilon1, epsilon2);
  fprintf (energy_file, "# nx, c_param, d_param = ");
  fprintf (energy_file, "%d, %g, %g \n", nx, c_param, d_param);

  fprintf (history_file,
	   "# Parameters: R, beta, gamma, epsilon1, epsilon2: ");
  fprintf (history_file, "%g, %g, %g, %g, %g\n", R_param, beta, gamma,
	   epsilon1, epsilon2);
  fprintf (history_file, "# nx, c_param, d_param = ");
  fprintf (history_file, "%d, %g, %g \n", nx, c_param, d_param);
}

/* ========================================================================= */

void Analyze_fin (void)
{
  fclose (energy_file);
  fclose (history_file);
}

/* ========================================================================= */
