/* ========================================================================= *
 * stepper.c -- time stepping routines for EZ-pipe (continuous version).  
 *
 * Copyright (C) 2011 Dwight Barkley                 
 *
 * ========================================================================= */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "pipe.h"
#include "stepper.h"

static double *Sigma_u;
static double *Sigma_q;

static void Impose_boundary_conditions (void);

/* ========================================================================= */

void Step (void)
{
  double  ukin, wkin;
  double  one_o_h = 1 / (grid_h);
  double  one_o_h2 = 1 / (grid_h * grid_h);
  register int i;

  Impose_boundary_conditions ();

  for (i = 1; i <= nx; i++) {
    Sigma_u[i] = -c_param * (u[i] - u[i - 1]) * one_o_h;
    Sigma_q[i] = (q[i + 1] - 2 * q[i] + q[i - 1]) * one_o_h2;
  }

  for (i = 1; i <= nx; i++) {
    ukin = epsilon1 * (1.0 - u[i]) - epsilon2 * q[i] * u[i];
    wkin = q[i] *
      (u[i] - (r_param + delta) * (q[i] - 1.) * (q[i] - 1.) + r_param - 1.);

    u[i] += dt * (ukin + Sigma_u[i]);
    q[i] += dt * (wkin + Sigma_q[i]);
  }

  /* Taking into account U d/dx assuming U = grid_h/dt */
  u[0] = u[nx];
  q[0] = q[nx];
  for (int i = nx - 1; i >= 0; i--) {
    u[i + 1] = u[i];
    q[i + 1] = q[i];
  }
}

/* ========================================================================= */

void Step_ini (void)
{
  Sigma_u = (double *) calloc (nx + 2, sizeof (double));
  Sigma_q = (double *) calloc (nx + 2, sizeof (double));

  Impose_boundary_conditions ();
}

/* ========================================================================= */

void Step_fin (void)
{
  free (Sigma_u);
  free (Sigma_q);
}

/* ========================================================================= */

void Impose_boundary_conditions (void)
{
  /* Average periodic directions (mostly this shouldn't be necessary) */
  u[1] = u[nx] = 0.5 * (u[nx] + u[1]);
  q[1] = q[nx] = 0.5 * (q[nx] + q[1]);

  /* Set fictitious points for periodic boundary conditions */
  u[0] = u[nx - 1];
  u[nx + 1] = u[2];

  q[0] = q[nx - 1];
  q[nx + 1] = q[2];
}

/* ========================================================================= */
